import telegram
from flask import Flask, request

from pemudapersis_bot.src.configs.environment import EnvConfig as Config
from pemudapersis_bot.src.receivers.telegram_bot import (
    handler,
    start_command,
    help_command,
    contact_command,
)

BOT = telegram.Bot(token=Config.BOT_TOKEN)
CERT = "cert.pem"
CERT_KEY = "key.pem"
CONTEXT = (CERT, CERT_KEY)
APP = Flask(__name__)


@APP.route("/pemudapersis-bot", methods=["POST"])
def respond():
    # retrieve the message in JSON and then transform it to Telegram object
    update = telegram.Update.de_json(request.get_json(force=True), BOT)

    if update.message.text == "/start":
        start_command(update)
    elif update.message.text == "/help":
        help_command(update)
    elif update.message.text == "/contact":
        contact_command(update)
    else:
        handler(update)

    return "ok"


@APP.route("/set_webhook", methods=["GET", "POST"])
def set_webhook():
    s = BOT.setWebhook(Config.WEBHOOK_URL, certificate=open(CERT, "rb"))
    if s:
        return f"Webhook setup OK : {Config.WEBHOOK_URL}"
    else:
        return f"Webhook setup failed : {Config.WEBHOOK_URL}"


@APP.route("/")
def index():
    return "<h1>Pemuda Persis Bot webhook is running</h1>"


def start():
    APP.run(
        threaded=True,
        port=Config.WEBHOOK_PORT,
        host=Config.WEBHOOK_HOST,
        ssl_context=CONTEXT,
    )


if __name__ == "__main__":
    start()



""" Configuration """
import logging
import os


class EnvConfig:
    BOT_DB = os.environ.get("BOT_DATABASE")
    BOT_TOKEN = os.environ.get("BOT_TOKEN")
    WEBHOOK_HOST = os.environ.get("WEBHOOK_HOST")
    WEBHOOK_URL = os.environ.get("WEBHOOK_URL", "0.0.0.0")
    WEBHOOK_PORT = os.environ.get("WEBHOOK_PORT", 8443)
    LOG_LEVEL = os.environ.get("BOT_LOG_LEVEL", logging.INFO)
    LOG_FORMAT = "%(asctime)s - %(levelname)s - %(message)s"

    OAUTH_URL = os.environ.get("OAUTH_URL", "http://206.189.147.212:8080/oauth/login")
    OAUTH_CREDS = os.environ.get("OAUTH_CREDS", "YW5hLWNsaWVudDphbmEtc2VjcmV0")
    OAUTH_USERNAME = os.environ.get("OAUTH_USERNAME", "99.0001")
    OAUTH_PASSWORD = os.environ.get("OAUTH_PASSWORD", "12345")
    OAUTH_GRANT_TYPE = os.environ.get("OAUTH_GRANT_TYPE", "password")
    API_ENDPOINT = os.environ.get("API_ENDPOINT", "http://206.189.147.212:8080")



import logging
from datetime import datetime
from telegram import Chat as TelegramChat
from pemudapersis_bot.src.configs.environment import EnvConfig as Conf
from pemudapersis_bot.src.models.user import BotUser
from pemudapersis_bot.src.models.chat import Chat

logging.basicConfig(format=Conf.LOG_FORMAT, level=Conf.LOG_LEVEL)


class ChatHandler:
    """Chat Handler"""

    def handler(self, chat: Chat):
        """Method for handing text chats"""

        # process reply
        reply: str = "Untuk sementara hanya berjalan di Group"
        if self:
            return reply

    def magic_keyword(self, keyword: str):
        magic_reply: str
        logging.info("Magic keyword : %s", keyword)
        magic_reply = "No such magic!"
        if self:
            return magic_reply

    @staticmethod
    def log_users(user: BotUser):
        """method for storing users to db"""
        chat_timestamp = datetime.now().strftime("%Y-%m-%d %H:%M:%S")
        log: list = [user.user_id, user.name, chat_timestamp]
        logging.info("Storing user %s", log)



""" Pemuda Persis bot on Telegram """

import logging
import os
from typing import Tuple, Optional
from telegram import Update, ForceReply, ParseMode, ChatMember, ChatMemberUpdated, Chat
from telegram.error import BadRequest
from telegram.ext import (
    Updater,
    CommandHandler,
    MessageHandler,
    Filters,
    CallbackContext,
    ChatMemberHandler,
)

from pemudapersis_bot.src.configs.environment import EnvConfig as Config
from pemudapersis_bot.src.models.chat import Chat as ChatData
from pemudapersis_bot.src.services.chat_handler import ChatHandler
from pemudapersis_bot.src.services.api_handler import APIHandler

# Enable logging
logging.basicConfig(format=Config.LOG_FORMAT, level=Config.LOG_LEVEL)

HANDLER = ChatHandler()
API_HANDLER = APIHandler()
HELP = """📜 Petunjuk penggunaan :

Petunjuk penggunaan Chatbot..
"""
NEW_MEMBERS = {}


def start_command(update: Update, context: CallbackContext = None) -> None:
    """Send a message when the command /start is issued."""
    user = update.effective_user
    update.message.reply_markdown_v2(
        f"Assalamualaikum {user.mention_markdown_v2()}\! Silahkan masukan NPA antum\.",
        reply_markup=ForceReply(selective=True),
    )


def help_command(update: Update, context: CallbackContext = None) -> None:
    """Send a message when the command /help is issued."""
    update.message.reply_text(HELP)


def contact_command(update: Update, context: CallbackContext = None) -> None:
    """Send a message when the command /contact is issued."""
    update.message.reply_text("Pertanyaan atau saran bisa hubungi @Admin")


def handler(update: Update, context: CallbackContext = None) -> None:
    """Method for handling messages"""
    logging.debug(NEW_MEMBERS)
    chat_user_id = update.message.from_user.id
    if (
        update.effective_chat.type in [Chat.GROUP, Chat.SUPERGROUP]
        and chat_user_id not in NEW_MEMBERS.keys()
    ):
        return
    elif chat_user_id in NEW_MEMBERS.keys() and len(update.message.text) != 7:
        update.message.reply_text("Format NPA salah. Silahkan coba lagi!!")
        return
    elif chat_user_id in NEW_MEMBERS.keys():
        npa_status = API_HANDLER.check_npa(npa=update.message.text)
        if npa_status == "Valid":
            NEW_MEMBERS.pop(chat_user_id)
            update.message.reply_text("NPA Valid. Selamat bergabung di Group Akhi!")
            return
        elif NEW_MEMBERS.get(chat_user_id) == 3:
            update.message.reply_text(
                "Tiga kali gagal verifikasi. Afwan, antum tidak diperbolehkan memposting apapun di Group ini (Banned)!"
            )
            update.effective_chat.ban_member(chat_user_id)
            return
        elif NEW_MEMBERS.get(chat_user_id) == 2:
            update.message.reply_text(
                "NPA Tidak Valid. Silahkan coba lagi! Kesempatan terakhir."
            )
            NEW_MEMBERS[chat_user_id] = NEW_MEMBERS.get(chat_user_id) + 1
            return
        else:
            update.message.reply_text("NPA Tidak Valid. Silahkan coba lagi!")
            NEW_MEMBERS[chat_user_id] = NEW_MEMBERS.get(chat_user_id) + 1
            return

    if update.message.text[0] == "!":
        reply = HANDLER.magic_keyword(update.message.text)
        update.message.reply_text(reply)
        return

    if update.message.from_user.username:
        sender_id = update.message.from_user.username
    else:
        sender_id = "unknown"

    chat = ChatData(
        chat_id=update.message.chat_id,
        sender_id=sender_id,
        sender=update.message.from_user.first_name,
        str_message=update.message.text,
    )
    reply = HANDLER.handler(chat)
    try:
        logging.debug("Sending message...")
        update.message.reply_text(reply)
    except BadRequest:
        update.message.reply_text("Afwan, ana belum paham soal itu.")
    logging.debug("Message sent..")


def extract_status_change(
    chat_member_update: ChatMemberUpdated,
) -> Optional[Tuple[bool, bool]]:
    """Takes a ChatMemberUpdated instance and extracts whether the 'old_chat_member' was a member
    of the chat and whether the 'new_chat_member' is a member of the chat. Returns None, if
    the status didn't change.
    """
    status_change = chat_member_update.difference().get("status")
    old_is_member, new_is_member = chat_member_update.difference().get(
        "is_member", (None, None)
    )

    if status_change is None:
        return None

    old_status, new_status = status_change
    was_member = old_status in [
        ChatMember.MEMBER,
        ChatMember.CREATOR,
        ChatMember.ADMINISTRATOR,
    ] or (old_status == ChatMember.RESTRICTED and old_is_member is True)
    is_member = new_status in [
        ChatMember.MEMBER,
        ChatMember.CREATOR,
        ChatMember.ADMINISTRATOR,
    ] or (new_status == ChatMember.RESTRICTED and new_is_member is True)

    return was_member, is_member


def greet_chat_members(update: Update, context: CallbackContext) -> None:
    """Greet new users in chats"""
    result = extract_status_change(update.chat_member)
    if result is None:
        return

    was_member, is_member = result
    member_name = update.chat_member.new_chat_member.user.mention_html()

    if not was_member and is_member:
        NEW_MEMBERS[update.chat_member.from_user.id] = 0
        update.effective_chat.send_message(
            f"Ahlan wa sahlan {member_name}. Untuk verifikasi, silahkan reply pesan ini dengan NPA antum!",
            parse_mode=ParseMode.HTML,
        )
    # elif was_member and not is_member:
    #     update.effective_chat.send_message(
    #         f"{member_name} is no longer with us. Thanks a lot, {cause_name} ...",
    #         parse_mode=ParseMode.HTML,
    #     )


def start_bot():
    """Start the bot."""
    # Create the Updater and pass your bot's token.
    if os.environ.get("BOT_TOKEN"):
        token = os.environ["BOT_TOKEN"]
    else:
        logging.error("BOT_TOKEN is not set!")
        return

    updater = Updater(token)
    # Get the dispatcher to register handlers
    dispatcher = updater.dispatcher

    # on different commands - answer in Telegram
    dispatcher.add_handler(CommandHandler("start", start_command))
    dispatcher.add_handler(CommandHandler("help", help_command))
    dispatcher.add_handler(CommandHandler("contact", contact_command))

    # in Group handlers
    dispatcher.add_handler(
        ChatMemberHandler(greet_chat_members, ChatMemberHandler.CHAT_MEMBER)
    )

    # on non command i.e message - echo the message on Telegram
    dispatcher.add_handler(MessageHandler(Filters.text & ~Filters.command, handler))

    # Start the Bot
    updater.start_polling(allowed_updates=Update.ALL_TYPES)

    # Run the bot until you press Ctrl-C or the process receives SIGINT,
    # SIGTERM or SIGABRT. This should be used most of the time, since
    # start_polling() is non-blocking and will stop the bot gracefully.
    updater.idle()


if __name__ == "__main__":
    start_bot()



# -*- coding: utf-8 -*-
""" Pemuda Persis bot """
from setuptools import setup, find_packages

requirements = [
    x.strip() for x in open('requirements.txt').readlines() if not x.startswith('#')]


setup(
    name="pemudapersis-bot",
    version="1.2.4",
    description="The official chatbot of Pemuda Persatuan Islam (PERSIS)",
    long_description=open("README.md").read().strip(),
    author="zamzambadruzaman",
    author_email="azzambz@pm.me",
    license="MIT",
    packages=find_packages(),
    include_package_data=True,
    zip_safe=False,
    install_requires=requirements,
    entry_points={
        "console_scripts": [
            "pemudapersis-bot = pemudapersis_bot.cli.cli:handle_commands",
        ],
    },
    python_requires=">=3.8",
)



"""" Command line interface for the bot"""
import logging
import os
import click
import colorama
from pyfiglet import figlet_format
from termcolor import colored
from pemudapersis_bot.src.configs.environment import EnvConfig as Conf
from pemudapersis_bot.src.receivers import telegram_webhook, telegram_bot

logging.basicConfig(format=Conf.LOG_FORMAT, level=Conf.LOG_LEVEL)
colorama.init()


@click.command()
@click.option("--poll", is_flag=True, help="Start the bot with polling method")
@click.option("--start", is_flag=True, help="Start the bot webhook")
def handle_commands(poll, start):
    """Main method for handling commands through CLI"""
    if not Conf.BOT_TOKEN:
        stdout(
            "ERROR: please set BOT_TOKEN to start the bot! ",
            color="red",
        )
        return
    stdout("\nPemuda Persis Bot", color="green", figlet_flag=True)
    if poll:
        stdout("Starting polling messages...", color="green")
        try:
            telegram_bot.start_bot()
            stdout("Server completed", color="green")
        except Exception as exc:
            stdout("Starting server failed! : {}".format(exc), color="red")

    if start:
        if not Conf.WEBHOOK_URL:
            stdout(
                "ERROR: WEBHOOK_URL needs to be set for starting webhook server! ",
                color="red",
            )
        if not Conf.WEBHOOK_HOST:
            stdout(
                "WARNING: WEBHOOK_HOST is not set, set default value : 0.0.0.0",
                color="yellow",
            )
        if not Conf.WEBHOOK_PORT:
            stdout(
                "WARNING: WEBHOOK_PORT is not set, set default value : 8443",
                color="yellow",
            )
        else:
            stdout(f"WEBHOOK_URL : {os.environ['WEBHOOK_URL']}", color="yellow")
        stdout("Starting Tabayyun webhook server...", color="green")
        telegram_webhook.start()
        try:
            telegram_webhook.start()
        except Exception as e:
            stdout("Starting server failed! : {}".format(e), color="red")


def stdout(string: str, color: str, font: str = "slant", figlet_flag: bool = False):
    """A helper method to print logs to stdout"""
    if not figlet_flag:
        logging.info(colored(string, color))
    else:
        logging.info(colored(figlet_format(string, font=font), color))


if __name__ == "__main__":
    handle_commands()



import requests
import logging
import json

from pemudapersis_bot.src.configs.environment import EnvConfig


class APIHandler:
    def __init__(self):
        self.config = EnvConfig()
        self.request_headers = {
            "Authorization": f"Basic {self.config.OAUTH_CREDS}",
            "Content-Type": "application/x-www-form-urlencoded",
        }
        self.access_token = None

    def check_npa(self, npa: str) -> str:
        api_url = f"{self.config.API_ENDPOINT}/v1/api/anggota/check"
        if not self.access_token:
            self.access_token = self.__get_access_token()
        self.request_headers = {
            "Authorization": f'bearer {self.access_token.get("access_token")}',
            "Content-Type": "application/json",
        }
        request_body = json.dumps({"npa": npa})
        try:
            response = requests.post(
                api_url, headers=self.request_headers, data=request_body
            )
            npa_data = response.json()
            logging.debug(npa_data)
            if npa_data.get("data"):
                return "Valid"
        except requests.RequestException as exp:
            logging.error(f"Failed to verify NPA : {exp}")
            return "Error"
        return "Invalid"

    def __get_access_token(self):
        logging.debug("Getting access token from Pemuda API..")
        response = {}
        request_body = {
            "username": self.config.OAUTH_USERNAME,
            "password": self.config.OAUTH_PASSWORD,
            "grant_type": self.config.OAUTH_GRANT_TYPE,
        }

        login_url = self.config.OAUTH_URL
        try:
            response = requests.post(
                login_url, data=request_body, headers=self.request_headers
            )
        except requests.RequestException as exp:
            logging.error(f"Failed to get access token : {exp}")
        return response.json()
